/*
 *   Copyright (c) International Business Machines  Corp., 2001
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Module: LvmUtils/helpers
 * File: get_container_handle.c
 */ 

#ifndef _LVM_HELP_GET_CONTAINER_HANDLE_
#define _LVM_HELP_GET_CONTAINER_HANDLE_ 1

#include "translate_name.c"

/* Function: get_container_handle
 *
 *	Find the container handle for the specified name in the specified
 *	plugin. Return standard errors for problems with GetList/GetInfo.
 *	Set container to NULL if the container doesn't exist.
 */
int get_container_handle(char		* name,
			object_handle_t	* container )
{
	char	cont_name[EVMS_VOLUME_NAME_SIZE] = {0};
	int	rc;

	translate_vg_name_to_container_name(name, cont_name);

	if ( opts.verbose ) {
		printf("%s -- Searching for container %s\n", cmd, cont_name);
	}

	// Get all containers from this plugin.
	rc = evms_get_object_handle_for_name(CONTAINER, cont_name, container);
	if (rc) {
		printf("%s -- Could not find handle for container %s\n", cmd, name);
		*container = 0;
		return EEXIST;
	}

	if ( opts.verbose ) {
		printf("%s -- Found container %s at handle %u\n", cmd, cont_name, *container);
	}

	return 0;
}

#endif

